/*
HDD Explorer Script Generator 
Copyright (c) 2006 Bruno Freitas
bootsector@ig.com.br
 
Permission is hereby granted, free of charge, to any person obtaining a copy of
this software and associated documentation files (the "Software"), to deal in
the Software without restriction, including without limitation the rights to
use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
of the Software, and to permit persons to whom the Software is furnished to do
so, subject to the following conditions:
 
The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.
 
THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
*/

import javax.swing.*;
import java.io.*;
import java.nio.channels.*;

public class Estrutura {
    
    private String dirEstrutura;
    private DefaultListModel listaAplicacoes;
    private static final String nova_linha = "\r\n"; // "\n" - Unix ou "\r\n" - Windows
    
    public Estrutura(String dirEstrutura, DefaultListModel listaAplicacoes) {
        this.dirEstrutura = dirEstrutura;
        this.listaAplicacoes = listaAplicacoes;
    }
    
    public void criaEstrutura() {
        // Limpa o diretorio - No funciona se diretorio estiver sendo "usado"
        //deletaDiretorio(new File(dirEstrutura + "/APPS/"));
        
        for(int i = 0; i < listaAplicacoes.size(); i++) {
            Aplicacao ap = (Aplicacao) listaAplicacoes.get(i);
            
            boolean criou = (new File(dirEstrutura + "/APPS/" + ap.getDiretorio())).mkdirs();
            
            //if (criou) {
            for(int j = 0; j < ap.getTotalArquivos(); j++) {
                copiaArquivo(ap.getArquivo(j), dirEstrutura + "/APPS/" + ap.getDiretorio() + "/" + ap.getArquivo(j).substring(ap.getArquivo(j).lastIndexOf(File.separatorChar)+1));
            }
            
            criaUninstall(ap);
            //}
        }
        
        criaArquivosDMS();
    }
    
    public void criaINSTALLCNF() {
        // INSTALL.CNF
        try {
            FileWriter arquivo = new FileWriter(dirEstrutura + "/" + "INSTALL.CNF");
            
            for(int i = 0; i < listaAplicacoes.size(); i++) {
                Aplicacao ap = (Aplicacao) listaAplicacoes.get(i);
                
                arquivo.write("[APP]" + nova_linha + "[HEADER]" + nova_linha + "[TITLE]"+nova_linha);
                arquivo.write(ap.getTitulo()+nova_linha);
                arquivo.write("[/TITLE]" + nova_linha + "[AUTHOR]"+nova_linha);
                arquivo.write(ap.getAutor()+nova_linha);
                arquivo.write("[/AUTHOR]" + nova_linha + "[DESC]"+nova_linha);
                arquivo.write(ap.getDescricao().replace("\n", "\\n")+nova_linha);
                arquivo.write("[/DESC]" + nova_linha + "[EXEC]"+nova_linha);
                arquivo.write("/" + ap.getDiretorio() + "/" + ap.getExecutavel().toUpperCase()+nova_linha);
                arquivo.write("[/EXEC]" + nova_linha + "[/HEADER]" + nova_linha + "[INSTALL]" + nova_linha + "[ACTION]"+nova_linha);
                arquivo.write("_MKDIR /" + ap.getDiretorio()+nova_linha);
                
                arquivo.write("[/ACTION]" + nova_linha);
                
                for(int j = 0; j < ap.getTotalArquivos(); j++) {
                    arquivo.write("[ACTION]" + nova_linha + "COPY /APPS/" + ap.getDiretorio() + "/" + ap.getArquivo(j).substring(ap.getArquivo(j).lastIndexOf(File.separatorChar)+1).toUpperCase() + " /" + ap.getDiretorio() + "/" + ap.getArquivo(j).substring(ap.getArquivo(j).lastIndexOf(File.separatorChar)+1).toUpperCase() + nova_linha + "[/ACTION]" + nova_linha);
                }
                
                arquivo.write("[/INSTALL]" + nova_linha + "[UNINSTALL]" + nova_linha);
                
                for(int j = 0; j < ap.getTotalArquivos(); j++) {
                    arquivo.write("[ACTION]" + nova_linha + "REMOVE /" + ap.getDiretorio() + "/" + ap.getArquivo(j).substring(ap.getArquivo(j).lastIndexOf(File.separatorChar)+1).toUpperCase() + nova_linha + "[/ACTION]" + nova_linha);
                }
                
                arquivo.write("[ACTION]"+nova_linha);
                
                arquivo.write("_RMDIR /" + ap.getDiretorio()+nova_linha);
                arquivo.write("[/ACTION]" + nova_linha + "[/UNINSTALL]" + nova_linha + "[/APP]"+nova_linha);
            }
            
            arquivo.close();
        } catch (Exception ex) {
            System.out.println(ex);
        }
    }
    
    private void copiaArquivo(String origem, String destino) {
        try {
            FileChannel srcChannel = new FileInputStream(origem).getChannel();
            
            String arq_destino = destino.substring(destino.lastIndexOf(File.separatorChar)+1);
            String dir_destino = destino.substring(0, destino.lastIndexOf(File.separatorChar)+1);

            FileChannel dstChannel = new FileOutputStream(dir_destino + arq_destino.toUpperCase()).getChannel();
            
            dstChannel.transferFrom(srcChannel, 0, srcChannel.size());
            
            srcChannel.close();
            dstChannel.close();
        } catch (IOException e) {
            System.out.println(e);
        }
    }
    
    private boolean deletaDiretorio(File dir) {
        if (dir.isDirectory()) {
            String[] children = dir.list();
            for (int i=0; i<children.length; i++) {
                boolean success = deletaDiretorio(new File(dir, children[i]));
                if (!success) {
                    return false;
                }
            }
        }
        
        return dir.delete();
    }
    
    private void criaArquivosDMS() {
        try {
            InputStream isHDD = getClass().getResourceAsStream("res/dms/HDD_EXP.ELF");
            InputStream isSYSTEM = getClass().getResourceAsStream("res/dms/SYSTEM.CNF");
            
            FileOutputStream osHDD = new FileOutputStream(dirEstrutura + "/HDD_EXP.ELF");
            FileOutputStream osSYSTEM = new FileOutputStream(dirEstrutura + "/SYSTEM.CNF");
            FileOutputStream osZZZ = new FileOutputStream(dirEstrutura + "/ZZZ.ZZZ");
            
            byte dummy[] = new byte[1048576];
            
            int b;
            
            b = isHDD.read();
            while (b != -1) {
                osHDD.write(b);
                b = isHDD.read();
            }
            
            b = isSYSTEM.read();
            while (b != -1) {
                osSYSTEM.write(b);
                b = isSYSTEM.read();
            }
            
            int j = 0;
            for (int i = 0; i < 1048576; i++) {
                dummy[i] = (byte) j;
                if(++j > 255) j = 0;
            }
            
            for (int i = 1; i <= 10; i++)
                osZZZ.write(dummy);
            
            isHDD.close();
            isSYSTEM.close();
            osHDD.close();
            osSYSTEM.close();
            osZZZ.close();
            
        } catch (Exception ex) {
            System.out.println(ex);
        }
        
    }
    
    private void criaUninstall(Aplicacao ap) {
        try {
            byte[] dummy = new byte[4228];
            RandomAccessFile arquivo = new RandomAccessFile(dirEstrutura + "/APPS/" + ap.getDiretorio() + "/" + "uninstall.cnf", "rw");
            int posicao;
            
            arquivo.write(dummy);
            
            arquivo.seek(0);
            arquivo.writeInt(toLittleEndian(ap.getTotalArquivos()+1));
            
            posicao = 4;
            
            for (int i = 0; i < ap.getTotalArquivos(); i++) {
                arquivo.seek(posicao);
                arquivo.writeInt(toLittleEndian(3));
                arquivo.writeBytes("/" + ap.getDiretorio() + "/" + ap.getArquivo(i).substring(ap.getArquivo(i).lastIndexOf(File.separatorChar)+1).toUpperCase());
                posicao += 132;
            }
            
            arquivo.seek(posicao);
            arquivo.writeInt(toLittleEndian(129));
            arquivo.writeBytes("/" + ap.getDiretorio());
            
            arquivo.close();
            
        } catch (Exception ex) {
            System.out.println(ex);
        }
    }
    
    private static int toLittleEndian(int v) {
        return (v >>> 24) | (v << 24) | ((v << 8) & 0x00FF0000) | ((v >> 8) & 0x0000FF00);
    }
}