/*
HDD Explorer Script Generator
Copyright (c) 2006 Bruno Freitas
bootsector@ig.com.br
 
Permission is hereby granted, free of charge, to any person obtaining a copy of
this software and associated documentation files (the "Software"), to deal in
the Software without restriction, including without limitation the rights to
use, copy, modify, merge, publish, distribute, sublicense, and/or sell copies
of the Software, and to permit persons to whom the Software is furnished to do
so, subject to the following conditions:
 
The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.
 
THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
 */

import javax.swing.*;
import javax.swing.border.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;

public class HDDScriptGen extends JFrame implements ActionListener {
    
    private JPanel pnlAplicacoes, pnlBtnAplicacoes, pnlGeracao, pnlDadosGeracao, pnlBtnGeracao;
    private JButton btnAppIncluir, btnAppEditar, btnAppExcluir, btnGerGeraScript, btnGerLocalizaPasta, btnSair, btnSobre, btnCima, btnBaixo, btnOrdena;
    private JList lstAplicacoes;
    private DefaultListModel lmAplicacoes;
    private JLabel lblGerLocal;
    private JTextField txtGerLocal;
    private JMenu mnuProjeto;
    private JMenuItem itmSalvarProjeto, itmAbrirProjeto;
    private JMenuBar barMenu;
    
    public HDDScriptGen() {
        super("HDD Script Generator v1.2b - Java Edition");
        
        Container container = getContentPane();
        container.setLayout(new BorderLayout());
        
        // Menu Principal
        barMenu = new JMenuBar();
        mnuProjeto = new JMenu("Project");
        mnuProjeto.setMnemonic('P');
        itmSalvarProjeto = new JMenuItem("Save project...");
        itmSalvarProjeto.addActionListener(this);
        itmAbrirProjeto = new JMenuItem("Open project...");
        itmAbrirProjeto.addActionListener(this);
        mnuProjeto.add(itmSalvarProjeto);
        mnuProjeto.add(itmAbrirProjeto);
        barMenu.add(mnuProjeto);
        setJMenuBar(barMenu);
        
        // Painel de Aplicaes e seu contedo
        
        // Aplicaes
        pnlAplicacoes = new JPanel(new BorderLayout());
        pnlAplicacoes.setBorder(new TitledBorder("Applications:"));
        
        lmAplicacoes = new DefaultListModel();
        lstAplicacoes = new JList(lmAplicacoes);
        lstAplicacoes.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
        lstAplicacoes.addMouseListener(new MouseListener());
        
        // Painel de botes de Aplicaes
        pnlBtnAplicacoes = new JPanel(new GridLayout(2, 3));
        
        btnAppIncluir = new JButton("Add");
        btnAppIncluir.setMnemonic('A');
        btnAppIncluir.setIcon(new ImageIcon(getClass().getResource("res/img/novo.png")));
        btnAppIncluir.addActionListener(this);
        pnlBtnAplicacoes.add(btnAppIncluir);
        
        btnAppEditar = new JButton("Edit");
        btnAppEditar.setMnemonic('E');
        btnAppEditar.setIcon(new ImageIcon(getClass().getResource("res/img/editar.png")));
        btnAppEditar.addActionListener(this);
        pnlBtnAplicacoes.add(btnAppEditar);
        
        btnAppExcluir = new JButton("Delete");
        btnAppExcluir.setMnemonic('D');
        btnAppExcluir.setIcon(new ImageIcon(getClass().getResource("res/img/deletar.png")));
        btnAppExcluir.addActionListener(this);
        pnlBtnAplicacoes.add(btnAppExcluir);
        
        btnCima = new JButton("Up");
        btnCima.setMnemonic('U');
        btnCima.setIcon(new ImageIcon(getClass().getResource("res/img/arrowup.png")));
        btnCima.addActionListener(this);
        pnlBtnAplicacoes.add(btnCima);
        
        btnBaixo = new JButton("Down");
        btnBaixo.setMnemonic('o');
        btnBaixo.setIcon(new ImageIcon(getClass().getResource("res/img/arrowdown.png")));
        btnBaixo.addActionListener(this);
        pnlBtnAplicacoes.add(btnBaixo);
        
        btnOrdena = new JButton("Sort");
        btnOrdena.setMnemonic('S');
        btnOrdena.setIcon(new ImageIcon(getClass().getResource("res/img/az.png")));
        btnOrdena.addActionListener(this);
        pnlBtnAplicacoes.add(btnOrdena);
        
        pnlAplicacoes.add(new JScrollPane(lstAplicacoes), BorderLayout.CENTER);
        pnlAplicacoes.add(pnlBtnAplicacoes, BorderLayout.SOUTH);
        
        // Gerao
        pnlGeracao = new JPanel(new BorderLayout());
        pnlGeracao.setBorder(new TitledBorder("File Structure and Script generation:"));
        
        pnlDadosGeracao = new JPanel(new GridLayout(2, 1));
        lblGerLocal = new JLabel("Generate File Structure in:");
        txtGerLocal = new JTextField(30);
        pnlDadosGeracao.add(lblGerLocal);
        pnlDadosGeracao.add(txtGerLocal);
        
        pnlBtnGeracao = new JPanel(new GridLayout(4, 1));
        
        btnGerLocalizaPasta = new JButton("Locate Folder:");
        btnGerLocalizaPasta.setMnemonic('L');
        btnGerLocalizaPasta.setIcon(new ImageIcon(getClass().getResource("res/img/localiza_pasta.png")));
        btnGerLocalizaPasta.addActionListener(this);
        pnlBtnGeracao.add(btnGerLocalizaPasta);
        
        btnGerGeraScript = new JButton("Generate File Structure + Script");
        btnGerGeraScript.setMnemonic('G');
        btnGerGeraScript.setIcon(new ImageIcon(getClass().getResource("res/img/gerar.png")));
        btnGerGeraScript.addActionListener(this);
        pnlBtnGeracao.add(btnGerGeraScript);
        
        btnSobre = new JButton("About this program...");
        btnSobre.setMnemonic('b');
        btnSobre.setIcon(new ImageIcon(getClass().getResource("res/img/sobre.png")));
        btnSobre.addActionListener(this);
        pnlBtnGeracao.add(btnSobre);
        
        btnSair = new JButton("Exit");
        btnSair.setMnemonic('x');
        btnSair.setIcon(new ImageIcon(getClass().getResource("res/img/cancelar.png")));
        btnSair.addActionListener(this);
        pnlBtnGeracao.add(btnSair);
        
        pnlGeracao.add(pnlDadosGeracao, BorderLayout.NORTH);
        pnlGeracao.add(pnlBtnGeracao, BorderLayout.SOUTH);
        
        // Adiciona os painis Aplicaes e Gerao na janela principal
        container.add(pnlAplicacoes, BorderLayout.CENTER);
        container.add(pnlGeracao, BorderLayout.SOUTH);
        
        setSize(360, 420);
        setIconImage(new ImageIcon(getClass().getResource("res/img/ps2.png")).getImage());
        setResizable(false);
        centerFrameOnScreen(this);
        setVisible(true);
    }
    
    public static void main(String args[]) {
        try {
            UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
        } catch(Exception e) {
            throw new RuntimeException(e);
        }
        
        Font defFont = new DefaultFont().getFont(Font.PLAIN, 11);
        
        UIManager.put("Menu.font", defFont);
        UIManager.put("MenuItem.font", defFont);
        UIManager.put("Label.font", defFont);
        UIManager.put("Button.font", defFont);
        UIManager.put("List.font", defFont);
        UIManager.put("TextField.font", defFont);
        UIManager.put("TitledBorder.font", defFont);
        
        HDDScriptGen app = new HDDScriptGen();
        
        app.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    }
    
    private void centerFrameOnScreen(JFrame frame) {
        Dimension d=Toolkit.getDefaultToolkit().getScreenSize();
        Dimension f=frame.getSize();
        
        frame.setBounds(Math.max(0, (d.width - f.width) / 2),
                Math.max(0, (d.height - f.height) / 2),
                f.width, f.height);
    }
    
    private void AppEditar() {
        if (lmAplicacoes.size() == 0) {
            JOptionPane.showMessageDialog(this, "There are not applications to be edited!", "Warning", JOptionPane.WARNING_MESSAGE);
            return;
        }
        
        Aplicacao ap = (Aplicacao) lstAplicacoes.getSelectedValue();
        
        if (ap == null) {
            JOptionPane.showMessageDialog(this, "You must select an application from the list first!", "Warning", JOptionPane.WARNING_MESSAGE);
            return;
        }
        
        FrmManAplicacao f = new FrmManAplicacao(this, "Edit application", ap, false);
        f.setVisible(true);
        f.dispose();
        
    }
    
    public void actionPerformed(ActionEvent e) {
        if (e.getSource() == btnAppIncluir) {
            Aplicacao ap = new Aplicacao();
            FrmManAplicacao f = new FrmManAplicacao(this, "Add application", ap, true);
            f.setVisible(true);
            f.dispose();
            
            if (ap.getTitulo().compareTo("") != 0) {
                lmAplicacoes.addElement(ap);
                lstAplicacoes.setSelectedValue(ap, true);
            }
        }
        
        if (e.getSource() == btnAppEditar) {
            AppEditar();
        }
        
        if (e.getSource() == btnAppExcluir) {
            if (lmAplicacoes.size() == 0) {
                JOptionPane.showMessageDialog(this, "There are not applications to be deleted!", "Warning", JOptionPane.WARNING_MESSAGE);
                return;
            }
            
            Aplicacao ap = (Aplicacao) lstAplicacoes.getSelectedValue();
            
            if (ap == null) {
                JOptionPane.showMessageDialog(this, "You must select an application from the list first!", "Warning", JOptionPane.WARNING_MESSAGE);
                return;
            }
            
            Object[] opcoes = {"Yes", "No"};
            if (JOptionPane.showOptionDialog(this, "Do you really want to delete the selected application?", "Confirmation", JOptionPane.YES_NO_OPTION, JOptionPane.QUESTION_MESSAGE, null, opcoes, opcoes[1]) == JOptionPane.YES_OPTION)
                lmAplicacoes.removeElement(ap);
        }
        
        if (e.getSource() == btnSair)
            System.exit(0);
        
        if (e.getSource() == btnSobre)
            JOptionPane.showMessageDialog(this, "Software developed by Bruno Freitas - bootsector@ig.com.br\n\n*** This software is FREEWARE and Open Source ***\n\nhttp://www.brunofreitas.com/" , "About this program...", JOptionPane.INFORMATION_MESSAGE);
        
        if (e.getSource() == btnGerGeraScript) {
            if (lmAplicacoes.size() == 0) {
                JOptionPane.showMessageDialog(this, "You must have at least one application in the list!", "Warning", JOptionPane.WARNING_MESSAGE);
                return;
            }
            
            if (txtGerLocal.getText().trim().length() == 0) {
                JOptionPane.showMessageDialog(this, "You must choose a directory for structure generation!", "Warning", JOptionPane.WARNING_MESSAGE);
                btnGerLocalizaPasta.requestFocus();
                return;
            }
            
            if (!(new File(txtGerLocal.getText())).exists()) {
                JOptionPane.showMessageDialog(this, "The chosen directory doesn't exist!", "Warning", JOptionPane.WARNING_MESSAGE);
                btnGerLocalizaPasta.requestFocus();
                return;
            }
            
            setCursor(new Cursor(Cursor.WAIT_CURSOR));
            
            Estrutura es = new Estrutura(txtGerLocal.getText(), lmAplicacoes);
            es.criaEstrutura();
            es.criaINSTALLCNF();
            AppList.GeraAppList(lmAplicacoes, txtGerLocal.getText() + "/APPS");
            
            setCursor(new Cursor(Cursor.DEFAULT_CURSOR));
            
            JOptionPane.showMessageDialog(this, "File Structure generated successfuly! Create an ISO by using CDGENPS2 and good installing!", "Information", JOptionPane.INFORMATION_MESSAGE);
        }
        
        if (e.getSource() == btnGerLocalizaPasta) {
            try {
                JFileChooser fc = new JFileChooser(new File(new File(".").getCanonicalPath()));
                fc.setDialogTitle("Choose folder...");
                fc.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
                
                if (fc.showOpenDialog(this) == JFileChooser.APPROVE_OPTION) {
                    txtGerLocal.setText(fc.getSelectedFile().getAbsolutePath());
                }
            } catch (Exception ex) {}
        }
        
        if (e.getSource() == btnCima)
            moveItemCima();
        
        if (e.getSource() == btnBaixo)
            moveItemBaixo();
        
        if (e.getSource() == btnOrdena)
            ordenaLista();
        
        if (e.getSource() == itmSalvarProjeto) {
            FileOutputStream fos = null;
            ObjectOutputStream oos = null;
            
            try {
                JFileChooser fc = new JFileChooser(new File(new File(".").getCanonicalPath()));
                fc.setFileFilter(new ProjetoFileFilter());
                fc.setDialogTitle("Save project...");
                
                if (fc.showSaveDialog(this) != JFileChooser.APPROVE_OPTION)
                    return;
                
                String nome_arquivo = fc.getSelectedFile().getAbsolutePath();
                if (!nome_arquivo.toUpperCase().endsWith(".HSG"))
                    nome_arquivo += ".hsg";
                
                fos = new FileOutputStream(nome_arquivo);
                oos = new ObjectOutputStream(fos);
                oos.writeObject(lmAplicacoes);
                oos.flush();
                oos.close();
                fos.close();
            } catch (Exception ex) {
                JOptionPane.showMessageDialog(this, "Error saving project!", "Error", JOptionPane.ERROR_MESSAGE);
            }
        }
        
        if (e.getSource() == itmAbrirProjeto) {
            FileInputStream fis = null;
            ObjectInputStream ois = null;
            DefaultListModel tmp;
            
            try {
                JFileChooser fc = new JFileChooser(new File(new File(".").getCanonicalPath()));
                fc.setFileFilter(new ProjetoFileFilter());
                fc.setDialogTitle("Open project...");
                
                if (fc.showOpenDialog(this) != JFileChooser.APPROVE_OPTION)
                    return;
                
                fis = new FileInputStream(fc.getSelectedFile().getAbsolutePath());
                ois = new ObjectInputStream(fis);
                tmp = (DefaultListModel) ois.readObject();
                
                lmAplicacoes.removeAllElements();
                
                for (int i = 0; i < tmp.size(); i++) {
                    lmAplicacoes.addElement(tmp.getElementAt(i));
                }
                
                
                ois.close();
                fis.close();
            } catch (Exception ex) {
                JOptionPane.showMessageDialog(this, "Error opening project!", "Error", JOptionPane.ERROR_MESSAGE);
            }
        }
    }
    
    private void moveItemCima() {
        int indice = lstAplicacoes.getSelectedIndex();
        if (indice == -1 || indice == 0) return;
        Aplicacao item1 = (Aplicacao) lmAplicacoes.getElementAt(indice-1);
        Aplicacao item2 = (Aplicacao) lmAplicacoes.getElementAt(indice);
        lmAplicacoes.setElementAt(item2, indice-1);
        lmAplicacoes.setElementAt(item1, indice);
        lstAplicacoes.setSelectedIndex(indice-1);
        lstAplicacoes.ensureIndexIsVisible(indice-1);
    }
    
    private void moveItemBaixo() {
        int indice = lstAplicacoes.getSelectedIndex();
        if (indice == -1 || indice == lmAplicacoes.size()-1)return;
        Aplicacao item1 = (Aplicacao) lmAplicacoes.getElementAt(indice);
        Aplicacao item2 = (Aplicacao) lmAplicacoes.getElementAt(indice+1);
        lmAplicacoes.setElementAt(item2, indice);
        lmAplicacoes.setElementAt(item1, indice+1);
        lstAplicacoes.setSelectedIndex(indice+1);
        lstAplicacoes.ensureIndexIsVisible(indice+1);
    }
    
    private void ordenaLista() {
        Comparator<Aplicacao> comp = new Comparator<Aplicacao>() {
            public int compare(Aplicacao a, Aplicacao b) {
                return (a.toString().compareToIgnoreCase(b.toString()));
            }
        };
        
        Vector<Aplicacao> lista = new Vector<Aplicacao>();
        
        for (int i = 0; i < lmAplicacoes.size(); i++)
            lista.add((Aplicacao)lmAplicacoes.get(i));
        
        Collections.sort(lista, comp);
        
        lmAplicacoes.removeAllElements();
        
        for (int i = 0; i < lista.size(); i++)
            lmAplicacoes.addElement(lista.get(i));
    }
    
    
    
    private class MouseListener extends MouseAdapter {
        public void mouseClicked(MouseEvent e) {
            if (e.getClickCount() == 2) {
                AppEditar();
            }
        }
    }
}
